@@ -1,6 +1,3 @@
-library(testthat)
-library_if_available(stringr)
-
 context("Joining strings")
 
 test_that("basic case works", {
